class Tiles {

    constructor(ctx, tilesFile, tilesWidth) {
        this.ctx = ctx;
        this.tiles = new Image();
        this.tiles.src = tilesFile;
        this.tilesWidth = tilesWidth;
        this.tilesPerRow = 0;

        this.tiles.onload = () => {

            fetch('./newWorld.json')
            .then(response => response.json())
            .then(data => {
                // Parse the JSON data
                tilemap = data;
        
                // Access tilemap data
                numBlocksWidth = tilemap.width;
                numBlocksHeight = tilemap.height;
                tileWidth = tilemap.tilewidth;
                tileHeight = tilemap.tileheight;
                layers = tilemap.layers;
        
                this.tilesPerRow = Math.floor(this.tiles.naturalWidth / this.tilesWidth);
                this.draw();
                
                mainCanvas.addEventListener('mousedown', (event) => tiles.handleMouseDown(event));
                mainCanvas.addEventListener('mousemove', (event) => tiles.handleMouseMove(event));
                mainCanvas.addEventListener('mouseup', () => tiles.handleMouseUp());

                mainCanvas.addEventListener('touchstart', tiles.handleTouchStart.bind(this), false);
                mainCanvas.addEventListener('touchmove', tiles.handleTouchMove.bind(this), false);
                mainCanvas.addEventListener('touchend', tiles.handleTouchEnd.bind(this), false);
        
          })
          .catch(error => console.error('Error loading tilemap:', error));

        };

        this.isDragging = false;
        this.dragStartX = 0;
        this.dragStartY = 0;
    }

    handleTouchStart(event) {
        // Get touch coordinates relative to canvas
        const touchX = event.touches[0].clientX - this.ctx.canvas.offsetLeft;
        const touchY = event.touches[0].clientY - this.ctx.canvas.offsetTop;

        // Store initial touch position
        this.dragStartX = touchX;
        this.dragStartY = touchY;

        this.isDragging = true;
    }

    handleTouchMove(event) {
        if (this.isDragging) {
            // Get touch coordinates relative to canvas
            const touchX = event.touches[0].clientX - this.ctx.canvas.offsetLeft;
            const touchY = event.touches[0].clientY - this.ctx.canvas.offsetTop;
    
            // Calculate distance moved
            const deltaX = (touchX - this.dragStartX) * 0.1; // Adjust sensitivity by changing multiplier (e.g., 0.2)
            const deltaY = (touchY - this.dragStartY) * 0.1; // Adjust sensitivity by changing multiplier (e.g., 0.2)
    
            // Update map coordinates based on touch movement
            if (deltaX > 0) {
                WorldCoord.x += 1;
            } else if (deltaX < 0) {
                WorldCoord.x -= 1;
            }
    
            if (deltaY > 0) {
                WorldCoord.y += 1;
            } else if (deltaY < 0) {
                WorldCoord.y -= 1;
            }
    
            // Ensure coordinates are within bounds
            WorldCoord.x = Math.max(0, Math.min(1000 - visibleWidthTiles, WorldCoord.x));
            WorldCoord.y = Math.max(0, Math.min(1000 - visibleHeightTiles, WorldCoord.y));
    
            // Store new touch position for next move event
            this.dragStartX = touchX;
            this.dragStartY = touchY;
    
            this.draw();
        }
    }

    handleTouchEnd(event) {
        this.isDragging = false;
    }


    handleMouseDown(event) {
        const rect = mainCanvas.getBoundingClientRect();
        this.dragStartX = event.clientX - rect.left;
        this.dragStartY = event.clientY - rect.top;
        this.isDragging = true;
    }

    // New method to handle mouse move event
    handleMouseMove(event) {
        if (this.isDragging) {
            const rect = mainCanvas.getBoundingClientRect();
            const mouseX = event.clientX - rect.left;
            const mouseY = event.clientY - rect.top;
            const deltaX = (mouseX - this.dragStartX) * 0.1; // Adjust sensitivity by changing multiplier (e.g., 0.2)
            const deltaY = (mouseY - this.dragStartY) * 0.1; // Adjust sensitivity by changing multiplier (e.g., 0.2)

            // Update map coordinates based on mouse movement
            if (deltaX > 0) {
                WorldCoord.x += 1;
            } else if (deltaX < 0) {
                WorldCoord.x -= 1;
            }

            if (deltaY > 0) {
                WorldCoord.y += 1;
            } else if (deltaY < 0) {
                WorldCoord.y -= 1;
            }

            // Ensure coordinates are within bounds
            WorldCoord.x = Math.max(0, Math.min(1000 - visibleWidthTiles, WorldCoord.x));
            WorldCoord.y = Math.max(0, Math.min(1000 - visibleHeightTiles, WorldCoord.y));

            this.dragStartX = mouseX;
            this.dragStartY = mouseY;
            this.draw();
        }
    }

    // New method to handle mouse up event
    handleMouseUp() {
        this.isDragging = false;
    }

    getTileIdFromCoords(x, y, tilesPerRow) {
        return y * tilesPerRow + x;
    }

    calculateTilePosition(tileIndex, tileSize) {
        tileIndex--;
        const x = Math.floor((tileIndex % this.tilesPerRow) * tileSize); // Round down x position
        const y = Math.floor(Math.floor(tileIndex / this.tilesPerRow) * tileSize); // Round down y position

        return { x, y };
    }

    draw() {
        for (let y = 0; y <= visibleHeightTiles; y++) {
            for (let x = 0; x <= visibleWidthTiles; x++) {
                const index = this.getTileIdFromCoords(x + WorldCoord.x, y + WorldCoord.y, numBlocksWidth);
                
                for(let l = 0; l < layers.length; l++) {
                    var tileNum = layers[l].data[index];
                    if (tileNum > 0) {
                        var tilePos = this.calculateTilePosition(tileNum, tileWidth);
                        ctx.drawImage(this.tiles, tilePos.x, tilePos.y, tileWidth, tileWidth, x * blockWidth, y * blockWidth, blockWidth, blockWidth);
                    }
                }
            }
        }
        this.updateCoordinatesDisplay();
    }

    updateCoordinatesDisplay() {
        ctx.font = '16px Arial';
        ctx.fillStyle = '#111111';
        ctx.fillText("World Coordinates: (" + WorldCoord.x + ", " + WorldCoord.y + ")", 11, 22);
        ctx.fillStyle = 'white';
        ctx.fillText("World Coordinates: (" + WorldCoord.x + ", " + WorldCoord.y + ")", 10, 20);
    }
}
